@@ -39,8 +39,10 @@ Arguments generally fall into one of several categories:
 import logging
 
 from botocore import xform_name
+from botocore.parameters import ListParameter, StructParameter
 
 from awscli.argprocess import unpack_cli_arg
+from awscli.schema import SchemaTransformer
 
 
 LOG = logging.getLogger('awscli.arguments')
@@ -178,7 +180,7 @@ class CustomArgument(BaseCLIArgument):
     def __init__(self, name, help_text='', dest=None, default=None,
                  action=None, required=None, choices=None, nargs=None,
                  cli_type_name=None, group_name=None, positional_arg=False,
-                 no_paramfile=False):
+                 no_paramfile=False, schema=None):
         self._name = name
         self._help = help_text
         self._dest = dest
@@ -193,6 +195,13 @@ class CustomArgument(BaseCLIArgument):
             choices = []
         self._choices = choices
         self.no_paramfile = no_paramfile
+        self._schema = schema
+
+        # If the top level element is a list then set nargs to
+        # accept multiple values seperated by a space.
+        if self._schema and self._schema.get('type', None) == 'array':
+            self._nargs = '+'
+
         # TODO: We should eliminate this altogether.
         # You should not have to depend on an argument_object
         # as part of the interface.  Currently the argprocess
@@ -228,6 +237,30 @@ class CustomArgument(BaseCLIArgument):
             kwargs['nargs'] = self._nargs
         parser.add_argument(cli_name, **kwargs)
 
+    def create_argument_object(self):
+        """
+        Create an argument object based on the JSON schema if one is set.
+        After calling this method, ``parameter.argument_object`` is available
+        e.g. for generating docs.
+        """
+        transformer = SchemaTransformer(self._schema)
+        transformed = transformer.transform()
+
+        # Set the parameter name from the parsed arg key name
+        transformed.update({'name': self.name})
+
+        LOG.debug('Custom parameter schema for {0}: {1}'.format(
+            self.name, transformed))
+
+        # Select the correct top level type
+        if transformed['type'] == 'structure':
+            self.argument_object = StructParameter(None, **transformed)
+        elif transformed['type'] == 'list':
+            self.argument_object = ListParameter(None, **transformed)
+        else:
+            raise ValueError('Invalid top level type {0}!'.format(
+                transformed['type']))
+
     @property
     def required(self):
         if self._required is None:
@@ -248,6 +281,8 @@ class CustomArgument(BaseCLIArgument):
             return self._cli_type_name
         elif self._action in ['store_true', 'store_false']:
             return 'boolean'
+        elif self.argument_object is not None:
+            return self.argument_object.type
         else:
             # Default to 'string' type if we don't have any
             # other info.
